package syhd.cas.config;

import org.jasig.cas.client.authentication.AuthenticationFilter;
import org.jasig.cas.client.configuration.ConfigurationKeys;
import org.jasig.cas.client.session.SingleSignOutFilter;
import org.jasig.cas.client.util.AssertionThreadLocalFilter;
import org.jasig.cas.client.util.HttpServletRequestWrapperFilter;
import org.jasig.cas.client.validation.Cas30ProxyReceivingTicketValidationFilter;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CorsFilter;
import syhd.cas.bean.CasClientProperties;
import syhd.cas.bean.CasRedirectFilter;
import syhd.cas.bean.SyhdCasAuthFilter;

import javax.annotation.Resource;

/**
 * @author zhangbaisen
 * @date 2022/2/21 18:47
 */
@EnableConfigurationProperties(CasClientProperties.class)
@Configuration
@ConditionalOnProperty(prefix = "cas",value = "enabled",matchIfMissing = true)
public class CasAutoConfig {
    
    @Resource
    private CasClientProperties casClientProperties;

    @Bean
    @ConditionalOnMissingBean
    @ConditionalOnProperty(prefix = "cas",value = "frontend-backend-enable")
    public FilterRegistrationBean<CorsFilter> corsFilter() {
        UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        CorsConfiguration config = new CorsConfiguration();
        config.setAllowCredentials(true);
        config.addAllowedOriginPattern("*");
        config.addAllowedHeader("*");
        config.addAllowedMethod("*");
        source.registerCorsConfiguration("/**", config);

        FilterRegistrationBean<CorsFilter> r = new FilterRegistrationBean<>();
        r.setFilter(new CorsFilter(source));
        r.addUrlPatterns("/*");
        r.setOrder(1);
        return r;
    }

    /** 单点登出，一定要放在其他filter之前 */
    @Bean
    public FilterRegistrationBean<SingleSignOutFilter> singleSignOutFilter(){
        SingleSignOutFilter filter = new SingleSignOutFilter();

        FilterRegistrationBean<SingleSignOutFilter> r = new FilterRegistrationBean<>();
        r.setFilter(filter);
        r.addUrlPatterns("/cas/logout");
        r.addInitParameter(ConfigurationKeys.CAS_SERVER_URL_PREFIX.getName(),casClientProperties.getServerUrlPrefix());
        r.setOrder(1);
        return r;
    }

    /** 认证 */
    @Bean
    public FilterRegistrationBean<AuthenticationFilter> acsAuthenticationFilter(){
        System.out.println("acsAuthenticationFilter --------------------------");
        AuthenticationFilter filter = new AuthenticationFilter();

        FilterRegistrationBean<AuthenticationFilter> r = new FilterRegistrationBean<>();
        r.setFilter(filter);
        r.addUrlPatterns("/*");
        r.addInitParameter(ConfigurationKeys.CAS_SERVER_LOGIN_URL.getName(), casClientProperties.getServerLoginUrl());
        r.addInitParameter(ConfigurationKeys.SERVER_NAME.getName(), casClientProperties.getServiceUrlPrefix());
        r.setOrder(4);
        return r;
    }

    /** 校验ticket */
    @Bean
    public FilterRegistrationBean<Cas30ProxyReceivingTicketValidationFilter> cas30ProxyReceivingTicketValidationFilter(){
        Cas30ProxyReceivingTicketValidationFilter filter = new Cas30ProxyReceivingTicketValidationFilter();

        FilterRegistrationBean<Cas30ProxyReceivingTicketValidationFilter> r = new FilterRegistrationBean<>();
        r.setFilter(filter);
        r.addUrlPatterns("/*");
        r.addInitParameter(ConfigurationKeys.CAS_SERVER_URL_PREFIX.getName(),casClientProperties.getServerUrlPrefix());
        r.addInitParameter(ConfigurationKeys.SERVER_NAME.getName(), casClientProperties.getServiceUrlPrefix());
        //获取到的assertion认证信息放入会话
        r.addInitParameter(ConfigurationKeys.USE_SESSION.getName(), "true");
        //认证成功后重定向到客户端的service地址
        r.addInitParameter(ConfigurationKeys.REDIRECT_AFTER_VALIDATION.getName(), "true");

        //true:调用 response.encodeURL 方法，该方法会在url后面加上 ;jsessionid
        r.addInitParameter(ConfigurationKeys.ENCODE_SERVICE_URL.getName(), "false");
        r.setOrder(6);
        return r;
    }

    @Bean
    @ConditionalOnProperty(prefix = "cas.requestWrapper",value = "enable")
    public FilterRegistrationBean<HttpServletRequestWrapperFilter> httpServletRequestWrapperFilter(){
        FilterRegistrationBean<HttpServletRequestWrapperFilter> r = new FilterRegistrationBean<>(new HttpServletRequestWrapperFilter());
        r.addUrlPatterns("/*");
        r.setOrder(8);
        return r;
    }

    @Bean
    @ConditionalOnProperty(prefix = "cas.threadLocal",value = "enable")
    public FilterRegistrationBean<AssertionThreadLocalFilter> assertionThreadLocalFilter(){
        FilterRegistrationBean<AssertionThreadLocalFilter> r = new FilterRegistrationBean<>(new AssertionThreadLocalFilter());
        r.addUrlPatterns("/*");
        r.setOrder(10);
        return r;
    }


    /** 前后端分离时，cas认证过滤器 */
    @Bean
    @ConditionalOnProperty(prefix = "cas",value = "frontend-backend-enable")
    public FilterRegistrationBean<SyhdCasAuthFilter> syhdCasAuthFilter(){
        SyhdCasAuthFilter f = new SyhdCasAuthFilter();
        f.setCasRedirect(casClientProperties.getFrontendBackendRedirect());
        f.setServerLoginUrl(casClientProperties.getServerLoginUrl());
        f.setServiceUrlPrefix(casClientProperties.getServiceUrlPrefix());

        FilterRegistrationBean<SyhdCasAuthFilter> r = new FilterRegistrationBean<>(f);
        r.addUrlPatterns("/*");
        r.setOrder(3);
        return r;
    }

    @Bean
    @ConditionalOnProperty(prefix = "cas",value = "frontend-backend-enable")
    public FilterRegistrationBean<CasRedirectFilter> casRedirectFilter(){
        FilterRegistrationBean<CasRedirectFilter> r = new FilterRegistrationBean<>(new CasRedirectFilter());
        r.addUrlPatterns(casClientProperties.getFrontendBackendRedirect());
        r.setOrder(10);
        return r;
    }



}
